import { OpenAPIV3 } from 'openapi-types';
export declare type OperationDefaults = {
    tags?: string[];
    summary?: string;
    description?: string;
    externalDocObject?: OpenAPIV3.ExternalDocumentationObject;
    operationId?: string;
    parameters?: (OpenAPIV3.ParameterObject | OpenAPIV3.ReferenceObject)[];
    requestBody?: OpenAPIV3.RequestBodyObject | OpenAPIV3.ReferenceObject;
    callbacks?: {
        [callback: string]: OpenAPIV3.ReferenceObject | OpenAPIV3.CallbackObject;
    };
    deprecated?: boolean;
};
export default class OperationBuilder {
    private static _defaults?;
    /**
     * Sets defaults for the operation object when using the build interface
     *
     * @param defaults The defaults to set - these are global to the OperationBuilder class
     */
    static defaults(defaults: OperationDefaults): void;
    /**
     * Start building a new Operation object
     *
     * @param responses The responses object - required by OpenApiv3
     * @returns OperationBuilder instances for method chaining
     */
    static new(responses: OpenAPIV3.ResponsesObject): OperationBuilder;
    private readonly _operation;
    private constructor();
    /**
     * Creates a deep copy of the current state of the operation and returns it.
     *
     * @returns A deep copy of the built operation object
     */
    build(): OpenAPIV3.OperationObject;
    /**
     * Shorthand for build() method
     *
     * @returns A deep copy of the built operation object
     */
    b(): OpenAPIV3.OperationObject;
    /**
     * Add a tags field to the Operation object
     *
     * @param tags The tags object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    tags: (tags: string[]) => OperationBuilder;
    /**
     * Add a summary field to the Operation object
     *
     * @param summary The summary string per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    summary: (summary: string) => OperationBuilder;
    /**
     * Add a description field to the Operation object
     *
     * @param description The description string per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    description: (description: string) => OperationBuilder;
    /**
     * Add a externalDocObject field to the Operation object
     *
     * @param externalDocObject The externalDocObject object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    externalDocs: (externalDocObject: OpenAPIV3.ExternalDocumentationObject) => OperationBuilder;
    /**
     * Add a operationId field to the Operation object.
     *
     * **Must be unique per document**
     *
     * @param operationId The operationId string per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    operationId: (operationId: string) => OperationBuilder;
    /**
     * Add a parameters field to the Operation object, will include defaults.
     *
     * @param parameters The parameters object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    parameters: (parameters: (OpenAPIV3.ParameterObject | OpenAPIV3.ReferenceObject)[], excludeDefault?: boolean) => OperationBuilder;
    /**
     * Add a requestBody field to the Operation object.
     *
     * @param requestBody The requestBody object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    requestBody: (requestBody: OpenAPIV3.RequestBodyObject | OpenAPIV3.ReferenceObject) => OperationBuilder;
    /**
     * Add a callbacks field to the Operation object.
     *
     * @param callbacks The callbacks object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    callbacks: (callbacks: {
        [callback: string]: OpenAPIV3.ReferenceObject | OpenAPIV3.CallbackObject;
    }) => OperationBuilder;
    /**
     * Add a deprecated field to the Operation object.
     *
     * @param deprecated The deprecated boolean per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    deprecated: (deprecated: boolean) => OperationBuilder;
    /**
     * Add a security field to the Operation object.
     *
     * @param security The security object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    security: (security: OpenAPIV3.SecurityRequirementObject[]) => OperationBuilder;
    /**
     * Add a servers field to the Operation object.
     *
     * @param servers The servers object per OpenApiv3 spec
     * @returns OperationBuilder instances for method chaining
     */
    servers: (servers: OpenAPIV3.ServerObject[]) => OperationBuilder;
}
